             .title      "Namcot 106/163 Reader (INES 19) Plugin"

;05/20/07 
;Written by BootGod

;03/03/09
;Added mirroring check. 175 and 340 chips have hardwired mirroring
             
             ;vectors for standard system calls

send_byte:	.equ 0200h
chk_vram:	.equ 0206h
chk_wram:	.equ 0209h
wr_ppu:		.equ 020ch

lo_address:	.equ 00e0h
lo_address_lo:	.equ 00e0h
lo_address_hi:	.equ 00e1h

hi_address:	.equ 00e2h
hi_address_lo:	.equ 00e2h
hi_address_hi:	.equ 00e3h

bank_lo:	.equ 00e4h
bank_hi:	.equ 00e5h

banksleft:	.equ 00e6h
curbank:	.equ 00e7h

numpages_lo:	.equ 00e8h
numpages_hi:	.equ 00e9h

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Namco 106/163/175/340 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 64 >> 4,512 >> 4
             .db 64 >> 3,256 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

;mirroring is either mapper ctrl or hardwired H/V
             lda #0e0h
             sta 0c000h
             sta 0c800h
             sta 0d000h
             sta 0d800h		;if mapr ctrl, this will set all NT's to 163's 1st 1K internal VRAM bank

             ldx #00h		;x = H mirroring
             
             lda #020h
             jsr wr_ppu		;set PPU to $2000

             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h		;write values 55h and AAh

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400

             lda 2007h
             cmp #055h
             bne got_mir

             lda 2007h
             cmp #0aah
             bne got_mir	;compare first bytes to see if they are the same

             lda #020h
             jsr wr_ppu		;reset PPU to $2000

             lda #0aah
             sta 2007h		;write AAh first this time
             lda #055h
             sta 2007h

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400 again

             lda 2007h
             cmp #0aah
             bne got_mir	;compare again

             inx		;is V mirroring

;check PPU $2800 as well, if that continues to be the same, it is mapper-controlled

             lda #028h
             jsr wr_ppu
             lda 2007h		;set PPU to $2800

             lda 2007h
             cmp #0aah
             bne got_mir	;compare, if different, is hard-wired

             lda 2007h
             cmp #055h
             bne got_mir

             ldx #4		;is mapr ctrl

got_mir:     txa
             jsr send_byte   ;send byte

             ;lda #04h             
             ;jsr send_byte	;mirroring is mapper controlled
             
             lda #1
             sta numpages_hi

             ldy #8
             jsr comp_pbank
             beq got_prg	;64K PRG

             asl numpages_hi
             ldy #16
             jsr comp_pbank
             beq got_prg	;128K PRG

             asl numpages_hi
             ldy #32
             jsr comp_pbank
             beq got_prg	;256K PRG

             asl numpages_hi	;512K PRG

got_prg:     lda #0		;send PRG header
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #1
             jsr send_byte

             lda #0
             sta curbank
             lda numpages_hi
             asl a
             asl a
             asl a
             sta banksleft

next_prg:    lda curbank	;set current bank
             sta 0e000h
             inc curbank

             lda #0		;set address to $8000
             sta lo_address_lo
             lda #080h
             sta lo_address_hi

             ldx #32		;set up for 8K dump
             ldy #0

dump_prg:    lda (lo_address),y
             jsr send_byte
             iny
             bne dump_prg
             inc lo_address_hi
             dex
             bne dump_prg
             dec banksleft
             bne next_prg
             

;read CHR stuff             
;try VRAM

             lda #00h
             sta 08000h

             jsr chk_vram
             bne no_vram
             jmp check_wram
                         
no_vram:     lda #1
             sta numpages_hi

             ldy #64
             jsr comp_cbank
             beq got_chr	;64K CHR

             asl numpages_hi
             ldy #128
             jsr comp_cbank
             beq got_chr	;128K CHR

             asl numpages_hi	;256K CHR
             
got_chr:     lda #0		;send CHR header
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #2
             jsr send_byte

             lda numpages_hi	;convert numpages into banksleft (numpages_hi << 6)
             asl a
             asl a
             asl a
             asl a
             asl a
             asl a
             sta banksleft

             lda #0
             sta curbank

             lda #0C0h		;set VROM only bits
             sta 0E800h

next_chr:    lda curbank
             sta 08000h

             lda #00h		;set PPU address to $0000
             jsr wr_ppu
             lda 2007h

             ldy #0		;set up for 1K dump
             ldx #4

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             inc curbank
             dec banksleft
             bne next_chr
             
;check for save-game RAM and back it up if it exists             

check_wram:  jsr chk_wram
             bne done

do_wram:     lda #32		;send header for 8K WRAM
             jsr send_byte
             lda #0
             jsr send_byte
             lda #3
             jsr send_byte

             lda #0		;set read address to $6000
             sta lo_address_lo
             lda #060h
             sta lo_address_hi

             ldy #0		;dump 8K WRAM
             ldx #32
            
sr_lp:       lda (lo_address),y
             jsr send_byte
             iny
             bne sr_lp
             inc lo_address_hi
             dex
             bne sr_lp
             
done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts


;y = bank to compare
;z=same
comp_pbank:  ldx #0
             stx bank_lo	;lower pointer
             sty bank_hi	;upper pointer
             sty banksleft	;# blocks to compare

cpb_loop:    lda #0		;init pointers to $8000 and $A000
             sta lo_address_lo
             sta hi_address_lo
             lda #080h
             sta lo_address_hi
             lda #0a0h
             sta hi_address_hi

             lda bank_lo
             sta 0e000h
             lda bank_hi
             sta 0e800h		;write in current banks

             ldx #32		;checking 8K banks
             ldy #0

cpb_loop2:   lda (lo_address),y
             cmp (hi_address),y
             bne cpb_diff
             iny
             bne cpb_loop2
             inc lo_address_hi
             inc hi_address_hi
             dex
             bne cpb_loop2
             inc bank_lo
             inc bank_hi
             dec banksleft
             bne cpb_loop

cpb_diff:    rts

;y = bank to compare
;z=same
;for CHR
comp_cbank:  ldx #0
             stx bank_lo	;lower pointer
             sty bank_hi	;upper pointer
             sty banksleft	;# blocks to compare

             lda #0c0h
             sta 0e800h

ccb_loop:    lda bank_lo
             sta 08000h
             lda bank_hi
             sta 08800h		;write in current banks

             ldx #4		;checking 1K banks
             lda #0
             sta curbank	;reset current bank

ccb_loop2:   ldy #0
             lda curbank
             sta 2006h
             sty 2006h
             lda 2007h		;set PPU address to x000h

ql:          lda 2007h
             sta 0300h,y
             iny
             bne ql		;load 256 bytes for testing

             lda curbank
             clc
             adc #4
             inc curbank
             sta 2006h
             sty 2006h
             lda 2007h		;set PPU address to x400h

ccb_loop3:   lda 2007h
             cmp 0300h,y
             bne ccb_diff
             iny
             bne ccb_loop3
             dex
             bne ccb_loop2
             inc bank_lo
             inc bank_hi
             dec banksleft
             bne ccb_loop

ccb_diff:    rts


             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
